@@ -13,21 +13,20 @@
 # under the License.
 
 import contextlib
+
 import mock
 import netaddr
-from oslo_config import cfg
-
-from neutron_lib.api.definitions import portbindings
-from neutron_lib import constants as n_const
-from neutron_lib import exceptions as n_exc
-from neutron_lib.plugins import directory
-from oslo_utils import uuidutils
-
 from neutron.db import l3_dvr_ha_scheduler_db
-from neutron.db import l3_hamode_db
 from neutron.extensions import external_net
 from neutron.tests.unit.extensions import test_l3
 from neutron.tests.unit.plugins.ml2 import test_plugin
+from neutron_lib.api.definitions import portbindings
+from neutron_lib import constants as n_const
+from neutron_lib import exceptions as n_exc
+from neutron_lib.plugins import constants as plugin_constants
+from neutron_lib.plugins import directory
+from oslo_config import cfg
+from oslo_utils import uuidutils
 
 from neutron_dynamic_routing.extensions import bgp
 from neutron_dynamic_routing.services.bgp import bgp_plugin
@@ -39,7 +38,6 @@ IMAGINARY = '2b2334c8-adfe-42d9-82c6-ad866c7fc5d8'  # non existent resource id
 
 
 class TestL3Plugin(test_l3.TestL3NatAgentSchedulingServicePlugin,
-                   l3_hamode_db.L3_HA_NAT_db_mixin,
                    l3_dvr_ha_scheduler_db.L3_DVR_HA_scheduler_db_mixin):
     pass
 
@@ -102,6 +100,14 @@ class BgpEntityCreationMixin(object):
                                 router_distributed=False):
         pass
 
+    @contextlib.contextmanager
+    def gw_network(self, name='ext-net', external=False, **kwargs):
+        with self.network(name=name, **kwargs) as gw_network:
+            if external:
+                self._update('networks', gw_network['network']['id'],
+                             {'network': {external_net.EXTERNAL: True}})
+            yield gw_network
+
     @contextlib.contextmanager
     def router(self, name='bgp-test-router', tenant_id=_uuid(),
                admin_state_up=True, **kwargs):
@@ -130,7 +136,8 @@ class BgpEntityCreationMixin(object):
         if address_scope:
             subnetpool_args['address_scope_id'] = address_scope['id']
 
-        with self.network() as ext_net, self.network() as int_net,\
+        with self.gw_network(external=True) as ext_net,\
+            self.network() as int_net,\
             self.subnetpool(prefixes, **subnetpool_args) as pool:
             subnetpool_id = pool['subnetpool']['id']
             gw_net_id = ext_net['network']['id']
@@ -142,8 +149,6 @@ class BgpEntityCreationMixin(object):
                             cidr=tenant_prefix,
                             subnetpool_id=subnetpool_id,
                             ip_version=tenant_ip_net.version) as int_subnet:
-                self._update('networks', gw_net_id,
-                             {'network': {external_net.EXTERNAL: True}})
                 ext_gw_info = {'network_id': gw_net_id}
                 with self.router(external_gateway_info=ext_gw_info,
                                  distributed=distributed,
@@ -157,21 +162,7 @@ class BgpEntityCreationMixin(object):
                     yield router, ext_net, int_net
 
 
-class BgpTests(test_plugin.Ml2PluginV2TestCase,
-               BgpEntityCreationMixin):
-    fmt = 'json'
-
-    def setup_parent(self):
-        self.l3_plugin = ('neutron_dynamic_routing.tests.unit.db.test_bgp_db.'
-                          'TestL3Plugin')
-        super(BgpTests, self).setup_parent()
-
-    def setUp(self):
-        super(BgpTests, self).setUp()
-        self.l3plugin = directory.get_plugin(n_const.L3)
-        self.bgp_plugin = bgp_plugin.BgpPlugin()
-        self.plugin = directory.get_plugin()
-
+class BgpTests(BgpEntityCreationMixin):
     @contextlib.contextmanager
     def subnetpool_with_address_scope(self, ip_version, prefixes=None,
                                       shared=False, admin=True,
@@ -408,7 +399,7 @@ class BgpTests(test_plugin.Ml2PluginV2TestCase,
         with self.subnetpool_with_address_scope(4,
                                                 prefixes=['8.0.0.0/8']) as sp:
             with self.bgp_speaker(sp['ip_version'], 1234) as speaker,\
-                self.network() as network:
+                self.gw_network() as network:
                 network_id = network['network']['id']
                 self.bgp_plugin.add_gateway_network(self.context,
                                                    speaker['id'],
@@ -420,21 +411,18 @@ class BgpTests(test_plugin.Ml2PluginV2TestCase,
 
     def test_create_bgp_speaker_with_network(self):
         with self.subnetpool_with_address_scope(4,
-                                                prefixes=['8.0.0.0/8']) as sp:
-            network = self.plugin.create_network(self.context,
-                                                {'network':
-                                                    {'name': 'test-net',
-                                                     'tenant_id': _uuid(),
-                                                     'admin_state_up': True,
-                                                     'shared': True}})
+                                                prefixes=['8.0.0.0/8']) as sp,\
+            self.gw_network(name='test-net', tenant_id=_uuid(),
+                            shared=True) as network:
+            network_id = network['network']['id']
             with self.bgp_speaker(sp['ip_version'], 1234,
-                                  networks=[network['id']]) as speaker:
+                                  networks=[network_id]) as speaker:
                 self.assertEqual(1, len(speaker['networks']))
-                self.assertTrue(network['id'] in speaker['networks'])
+                self.assertTrue(network_id in speaker['networks'])
 
     def test_remove_gateway_network(self):
-        with self.network() as network1,\
-            self.network() as network2,\
+        with self.gw_network() as network1,\
+            self.gw_network() as network2,\
             self.subnetpool_with_address_scope(4,
                                                prefixes=['8.0.0.0/8']) as sp:
             network1_id = network1['network']['id']
@@ -474,7 +462,7 @@ class BgpTests(test_plugin.Ml2PluginV2TestCase,
                                                 prefixes=['8.0.0.0/8']) as sp:
             with self.bgp_speaker(sp['ip_version'], 1234) as speaker1,\
                 self.bgp_speaker(sp['ip_version'], 4321) as speaker2,\
-                self.network() as network:
+                self.gw_network() as network:
                 network_id = network['network']['id']
                 self.bgp_plugin.add_gateway_network(self.context,
                                                    speaker1['id'],
@@ -512,8 +500,8 @@ class BgpTests(test_plugin.Ml2PluginV2TestCase,
             self.subnetpool_with_address_scope(4,
                                                prefixes=prefixes3,
                                                tenant_id=tenant_id) as sp3,\
-            self.network() as network1, self.network() as network2,\
-            self.network() as network3:
+            self.gw_network() as network1, self.gw_network() as network2,\
+            self.gw_network() as network3:
             network1_id = network1['network']['id']
             network2_id = network2['network']['id']
             network3_id = network3['network']['id']
@@ -724,12 +712,11 @@ class BgpTests(test_plugin.Ml2PluginV2TestCase,
                                        prefixes=[binding_cidr]) as ext_pool,\
             self.subnetpool_with_address_scope(6, tenant_id=_uuid(),
                                        prefixes=[tenant_cidr]) as int_pool,\
-            self.network() as ext_net, self.network() as int_net:
+            self.gw_network(external=True) as ext_net,\
+            self.network() as int_net:
             gw_net_id = ext_net['network']['id']
             ext_pool_id = ext_pool['id']
             int_pool_id = int_pool['id']
-            self._update('networks', gw_net_id,
-                         {'network': {external_net.EXTERNAL: True}})
             with self.subnet(cidr=None,
                              subnetpool_id=ext_pool_id,
                              network=ext_net,
@@ -1096,11 +1083,10 @@ class BgpTests(test_plugin.Ml2PluginV2TestCase,
                 self.assertTrue(fip_prefix_found)
 
     def test__bgp_speakers_for_gateway_network_by_ip_version(self):
-        with self.network() as ext_net, self.bgp_speaker(6, 1234) as s1,\
+        with self.gw_network(external=True) as ext_net,\
+            self.bgp_speaker(6, 1234) as s1,\
             self.bgp_speaker(6, 4321) as s2:
             gw_net_id = ext_net['network']['id']
-            self._update('networks', gw_net_id,
-                         {'network': {external_net.EXTERNAL: True}})
             self.bgp_plugin.add_gateway_network(self.context,
                                                 s1['id'],
                                                 {'network_id': gw_net_id})
@@ -1114,11 +1100,10 @@ class BgpTests(test_plugin.Ml2PluginV2TestCase,
             self.assertEqual(2, len(speakers))
 
     def test__bgp_speakers_for_gateway_network_by_ip_version_no_binding(self):
-        with self.network() as ext_net, self.bgp_speaker(6, 1234),\
+        with self.gw_network(external=True) as ext_net,\
+            self.bgp_speaker(6, 1234),\
             self.bgp_speaker(6, 4321):
             gw_net_id = ext_net['network']['id']
-            self._update('networks', gw_net_id,
-                         {'network': {external_net.EXTERNAL: True}})
             speakers = self.bgp_plugin._bgp_speakers_for_gw_network_by_family(
                                                                  self.context,
                                                                  gw_net_id,
@@ -1316,3 +1301,19 @@ class BgpTests(test_plugin.Ml2PluginV2TestCase,
 
     def test__get_fip_next_hop_dvr(self):
         self._test__get_fip_next_hop(distributed=True)
+
+
+class Ml2BgpTests(test_plugin.Ml2PluginV2TestCase,
+                  BgpTests):
+    fmt = 'json'
+
+    def setup_parent(self):
+        self.l3_plugin = ('neutron_dynamic_routing.tests.unit.db.test_bgp_db.'
+                          'TestL3Plugin')
+        super(Ml2BgpTests, self).setup_parent()
+
+    def setUp(self):
+        super(Ml2BgpTests, self).setUp()
+        self.l3plugin = directory.get_plugin(plugin_constants.L3)
+        self.bgp_plugin = bgp_plugin.BgpPlugin()
+        self.plugin = directory.get_plugin()
